@@ -179,6 +179,77 @@ static jboolean JNICALL NativeSaveState(JNIEnv *env, jobject this, jstring fname
 static jint JNICALL NativeRunAtariProgram(JNIEnv *env, jobject this,
 												  jstring img, jint drv, jint reboot)
 {
+	static char const * const cart_descriptions[CARTRIDGE_LAST_SUPPORTED + 1] = {
+		NULL,
+		CARTRIDGE_STD_8_DESC,
+		CARTRIDGE_STD_16_DESC,
+		CARTRIDGE_OSS_034M_16_DESC,
+		CARTRIDGE_5200_32_DESC,
+		CARTRIDGE_DB_32_DESC,
+		CARTRIDGE_5200_EE_16_DESC,
+		CARTRIDGE_5200_40_DESC,
+		CARTRIDGE_WILL_64_DESC,
+		CARTRIDGE_EXP_64_DESC,
+		CARTRIDGE_DIAMOND_64_DESC,
+		CARTRIDGE_SDX_64_DESC,
+		CARTRIDGE_XEGS_32_DESC,
+		CARTRIDGE_XEGS_07_64_DESC,
+		CARTRIDGE_XEGS_128_DESC,
+		CARTRIDGE_OSS_M091_16_DESC,
+		CARTRIDGE_5200_NS_16_DESC,
+		CARTRIDGE_ATRAX_128_DESC,
+		CARTRIDGE_BBSB_40_DESC,
+		CARTRIDGE_5200_8_DESC,
+		CARTRIDGE_5200_4_DESC,
+		CARTRIDGE_RIGHT_8_DESC,
+		CARTRIDGE_WILL_32_DESC,
+		CARTRIDGE_XEGS_256_DESC,
+		CARTRIDGE_XEGS_512_DESC,
+		CARTRIDGE_XEGS_1024_DESC,
+		CARTRIDGE_MEGA_16_DESC,
+		CARTRIDGE_MEGA_32_DESC,
+		CARTRIDGE_MEGA_64_DESC,
+		CARTRIDGE_MEGA_128_DESC,
+		CARTRIDGE_MEGA_256_DESC,
+		CARTRIDGE_MEGA_512_DESC,
+		CARTRIDGE_MEGA_1024_DESC,
+		CARTRIDGE_SWXEGS_32_DESC,
+		CARTRIDGE_SWXEGS_64_DESC,
+		CARTRIDGE_SWXEGS_128_DESC,
+		CARTRIDGE_SWXEGS_256_DESC,
+		CARTRIDGE_SWXEGS_512_DESC,
+		CARTRIDGE_SWXEGS_1024_DESC,
+		CARTRIDGE_PHOENIX_8_DESC,
+		CARTRIDGE_BLIZZARD_16_DESC,
+		CARTRIDGE_ATMAX_128_DESC,
+		CARTRIDGE_ATMAX_1024_DESC,
+		CARTRIDGE_SDX_128_DESC,
+		CARTRIDGE_OSS_8_DESC,
+		CARTRIDGE_OSS_043M_16_DESC,
+		CARTRIDGE_BLIZZARD_4_DESC,
+		CARTRIDGE_AST_32_DESC,
+		CARTRIDGE_ATRAX_SDX_64_DESC,
+		CARTRIDGE_ATRAX_SDX_128_DESC,
+		CARTRIDGE_TURBOSOFT_64_DESC,
+		CARTRIDGE_TURBOSOFT_128_DESC,
+		CARTRIDGE_ULTRACART_32_DESC,
+		CARTRIDGE_LOW_BANK_8_DESC,
+		CARTRIDGE_SIC_128_DESC,
+		CARTRIDGE_SIC_256_DESC,
+		CARTRIDGE_SIC_512_DESC,
+		CARTRIDGE_STD_2_DESC,
+		CARTRIDGE_STD_4_DESC,
+		CARTRIDGE_RIGHT_4_DESC,
+		CARTRIDGE_BLIZZARD_32_DESC,
+		CARTRIDGE_MEGAMAX_2048_DESC,
+		CARTRIDGE_THECART_128M_DESC,
+		CARTRIDGE_MEGA_4096_DESC,
+		CARTRIDGE_MEGA_2048_DESC,
+		CARTRIDGE_THECART_32M_DESC,
+		CARTRIDGE_THECART_64M_DESC,
+		CARTRIDGE_XEGS_8F_64_DESC
+	};
+
 	const jbyte *img_utf = NULL;
 	int ret = 0, r, kb, i, cnt = 0;
 	jclass cls, scls;
@@ -209,7 +280,7 @@ static jint JNICALL NativeRunAtariProgram(JNIEnv *env, jobject this,
 				str = (*env)->NewStringUTF(env, tmp);
 				(*env)->SetObjectArrayElement(env, xarr, 0, str);
 				(*env)->DeleteLocalRef(env, str);
-				str = (*env)->NewStringUTF(env, CARTRIDGE_TextDesc[i]);
+				str = (*env)->NewStringUTF(env, cart_descriptions[i]);
 				(*env)->SetObjectArrayElement(env, xarr, 1, str);
 				(*env)->DeleteLocalRef(env, str);
 				(*env)->SetObjectArrayElement(env, arr, cnt++, xarr);
