@@ -2,7 +2,7 @@
  * atari.c - main high-level routines
  *
  * Copyright (c) 1995-1998 David Firth
- * Copyright (c) 1998-2008 Atari800 development team (see DOC/CREDITS)
+ * Copyright (c) 1998-2014 Atari800 development team (see DOC/CREDITS)
  *
  * This file is part of the Atari800 emulator project which emulates
  * the Atari 400, 800, 800XL, 130XE, and 5200 8-bit computers.
@@ -32,16 +32,6 @@
 #ifdef HAVE_SIGNAL_H
 #include <signal.h>
 #endif
-#ifdef TIME_WITH_SYS_TIME
-# include <sys/time.h>
-# include <time.h>
-#else
-# ifdef HAVE_SYS_TIME_H
-#  include <sys/time.h>
-# elif defined(HAVE_TIME_H)
-#  include <time.h>
-# endif
-#endif
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
@@ -67,6 +57,7 @@
 
 #include "akey.h"
 #include "antic.h"
+#include "artifact.h"
 #include "atari.h"
 #include "binload.h"
 #include "cartridge.h"
@@ -177,7 +168,6 @@ int Atari800_auto_frameskip = FALSE;
 
 #ifdef BENCHMARK
 static double benchmark_start_time;
-static double Atari_time(void);
 #endif
 
 int emuos_mode = 1;	/* 0 = never use EmuOS, 1 = use EmuOS if real OS not available, 2 = always use EmuOS */
@@ -264,17 +254,7 @@ void Atari800_Coldstart(void)
 	MEMORY_dPutByte(0x244, 1);
 	/* handle Option key (disable BASIC in XL/XE)
 	   and Start key (boot from cassette) */
-	GTIA_consol_index = 2;
-	GTIA_consol_table[2] = 0x0f;
-	if (Atari800_builtin_basic && Atari800_disable_basic && !BINLOAD_loading_basic) {
-		/* Only for XL/XE - hold Option during reboot. */
-		GTIA_consol_table[2] &= ~INPUT_CONSOL_OPTION;
-	}
-	if (CASSETTE_hold_start && Atari800_machine_type != Atari800_MACHINE_5200) {
-		/* Only for the computers - hold Start during reboot */
-		GTIA_consol_table[2] &= ~INPUT_CONSOL_START;
-	}
-	GTIA_consol_table[1] = GTIA_consol_table[2];
+	GTIA_consol_override = 2;
 #ifdef AF80
 	if (AF80_enabled) {
 		AF80_Reset();
@@ -735,6 +715,7 @@ int Atari800_Initialise(int *argc, char *argv[])
 	if (!SYSROM_Initialise(argc, argv)
 #if !defined(BASIC) && !defined(CURSES_BASIC)
 		|| !Colours_Initialise(argc, argv)
+		|| !ARTIFACT_Initialise(argc, argv)
 #endif
 		|| !Devices_Initialise(argc, argv)
 		|| !RTIME_Initialise(argc, argv)
@@ -870,8 +851,8 @@ int Atari800_Initialise(int *argc, char *argv[])
 	/* Load state file */
 	if (state_file != NULL) {
 		if (StateSav_ReadAtariState(state_file, "rb"))
-			/* Don't press Option */
-			GTIA_consol_table[1] = GTIA_consol_table[2] = 0x0f;
+			/* Don't press Start nor Option */
+			GTIA_consol_override = 0;
 	}
 #endif
 
@@ -891,8 +872,25 @@ int Atari800_Initialise(int *argc, char *argv[])
 #endif /* __PLUS */
 
 #ifdef BENCHMARK
-	benchmark_start_time = Atari_time();
+	benchmark_start_time = Util_time();
 #endif
+
+#if defined (SOUND) && defined(SOUND_THIN_API)
+	if (Sound_enabled) {
+		/* Up to this point the Sound_enabled flag indicated that we _want_ to
+		   enable sound. From now on, the flag will indicate whether audio
+		   output is enabled and working. So, since the sound output was not
+		   yet initiated, we set the flag accordingly. */
+		Sound_enabled = FALSE;
+		/* Don't worry, Sound_Setup() will set Sound_enabled back to TRUE if
+		   it opens audio output successfully. But Sound_Setup() relies on the
+		   flag being set if and only if audio output is active. */
+		if (Sound_Setup())
+			/* Start sound if opening audio output was successful. */
+				Sound_Continue();
+	}
+#endif /* defined (SOUND) && defined(SOUND_THIN_API) */
+
 	return TRUE;
 }
 
@@ -948,6 +946,9 @@ int Atari800_Exit(int run_monitor)
 
 		/* Cleanup functions, in reverse order as the init functions in
 		   Atari800_Initialise(). */
+#ifdef SOUND
+		Sound_Exit();
+#endif
 #if SUPPORTS_CHANGE_VIDEOMODE
 		VIDEOMODE_Exit();
 #endif
@@ -987,80 +988,6 @@ void Atari800_ErrExit(void)
 }
 
 #ifndef __PLUS
-
-#ifdef PS2
-
-double Atari_time(void);
-void Atari_sleep(double s);
-
-#else /* PS2 */
-
-static double Atari_time(void)
-{
-#ifdef SDL
-	return SDL_GetTicks() * 1e-3;
-#elif defined(HAVE_WINDOWS_H)
-	return GetTickCount() * 1e-3;
-#elif defined(DJGPP)
-	/* DJGPP has gettimeofday, but it's not more accurate than uclock */
-	return uclock() * (1.0 / UCLOCKS_PER_SEC);
-#elif defined(HAVE_GETTIMEOFDAY)
-	struct timeval tp;
-	gettimeofday(&tp, NULL);
-	return tp.tv_sec + 1e-6 * tp.tv_usec;
-#elif defined(HAVE_UCLOCK)
-	return uclock() * (1.0 / UCLOCKS_PER_SEC);
-#elif defined(HAVE_CLOCK)
-	return clock() * (1.0 / CLK_TCK);
-#else
-#error No function found for Atari_time()
-#endif
-}
-
-/* FIXME: Ports should use SUPPORTS_PLATFORM_SLEEP and SUPPORTS_PLATFORM_TIME */
-/* and not this mess */
-#ifndef SUPPORTS_PLATFORM_SLEEP
-
-static void Atari_sleep(double s)
-{
-	if (s > 0) {
-#ifdef HAVE_WINDOWS_H
-		Sleep((DWORD) (s * 1e3));
-#elif defined(DJGPP)
-		/* DJGPP has usleep and select, but they don't work that good */
-		/* XXX: find out why */
-		double curtime = Atari_time();
-		while ((curtime + s) > Atari_time());
-#elif defined(HAVE_NANOSLEEP)
-		struct timespec ts;
-		ts.tv_sec = 0;
-		ts.tv_nsec = s * 1e9;
-		nanosleep(&ts, NULL);
-#elif defined(HAVE_USLEEP)
-		usleep(s * 1e6);
-#elif defined(__BEOS__)
-		/* added by Walter Las for BeOS */
-		snooze(s * 1e6);
-#elif defined(__EMX__)
-		/* added by Brian Smith for os/2 */
-		DosSleep(s);
-#elif defined(HAVE_SELECT)
-		/* linux */
-		struct timeval tp;
-		tp.tv_sec = 0;
-		tp.tv_usec = s * 1e6;
-		select(1, NULL, NULL, NULL, &tp);
-#else
-		double curtime = Atari_time();
-		while ((curtime + s) > Atari_time());
-#endif
-	}
-}
-
-#endif /* SUPPORTS_PLATFORM_SLEEP */
-
-#endif /* PS2 */
-
 static void autoframeskip(double curtime, double lasttime)
 {
 	static int afs_lastframe = 0, afs_discard = 0;
@@ -1091,7 +1018,7 @@ static void autoframeskip(double curtime, double lasttime)
 
 		afs_sleeptime = 0.0;
 		afs_lastframe = Atari800_nframes;
-		afs_lasttime = Atari_time();
+		afs_lasttime = Util_time();
 	}
 }
 
@@ -1102,22 +1029,18 @@ void Atari800_Sync(void)
 	double curtime;
 
 #ifdef SYNCHRONIZED_SOUND
-	deltatime *= PLATFORM_AdjustSpeed();
+	deltatime *= Sound_AdjustSpeed();
 #endif
 #ifdef ALTERNATE_SYNC_WITH_HOST
 	if (! UI_is_active)
 		deltatime *= Atari800_refresh_rate;
 #endif
 	lasttime += deltatime;
-	curtime = Atari_time();
+	curtime = Util_time();
 	if (Atari800_auto_frameskip)
 		autoframeskip(curtime, lasttime);
-#ifdef SUPPORTS_PLATFORM_SLEEP
-	PLATFORM_Sleep(lasttime - curtime);
-#else
-	Atari_sleep(lasttime - curtime);
-#endif
-	curtime = Atari_time();
+	Util_sleep(lasttime - curtime);
+	curtime = Util_time();
 
 	if ((lasttime + deltatime) < curtime)
 		lasttime = curtime;
@@ -1352,7 +1275,7 @@ void Atari800_Frame(void)
 #else
 		ANTIC_Frame(TRUE);
 		INPUT_DrawMousePointer();
-		Screen_DrawAtariSpeed(Atari_time());
+		Screen_DrawAtariSpeed(Util_time());
 		Screen_DrawDiskLED();
 		Screen_Draw1200LED();
 #endif /* CURSES_BASIC */
@@ -1378,7 +1301,7 @@ void Atari800_Frame(void)
 	Atari800_nframes++;
 #ifdef BENCHMARK
 	if (Atari800_nframes >= BENCHMARK) {
-		double benchmark_time = Atari_time() - benchmark_start_time;
+		double benchmark_time = Util_time() - benchmark_start_time;
 		Atari800_ErrExit();
 		printf("%d frames emulated in %.2f seconds\n", BENCHMARK, benchmark_time);
 		exit(0);
@@ -1388,7 +1311,20 @@ void Atari800_Frame(void)
 #ifdef ALTERNATE_SYNC_WITH_HOST
 	if (refresh_counter == 0)
 #endif
-		if (Atari800_turbo == FALSE) Atari800_Sync();
+		if (Atari800_turbo) {
+			/* No need to draw Atari frames with frequency higher than display
+			   refresh rate. */
+			static double last_display_screen_time = 0.0;
+			static double const limit = 1.0 / 60.0; /* refresh every 1/60 s */
+			/* TODO Actually sync the limit with the display refresh rate. */
+			double cur_time = Util_time();
+			if (cur_time - last_display_screen_time > limit)
+				last_display_screen_time = cur_time;
+			else
+				Atari800_display_screen = FALSE;
+		}
+		else
+			Atari800_Sync();
 #endif /* BENCHMARK */
 }
 
@@ -1527,13 +1463,18 @@ void Atari800_SetTVMode(int mode)
 		Atari800_tv_mode = mode;
 #if !defined(BASIC) && !defined(CURSES_BASIC)
 		Colours_SetVideoSystem(mode);
+		ARTIFACT_SetTVMode(mode);
 #endif
 #if SUPPORTS_CHANGE_VIDEOMODE
 		VIDEOMODE_SetVideoSystem(mode);
 #endif
-#if defined(SOUND) && defined(SUPPORTS_SOUND_REINIT)
+#ifdef SOUND
+#ifdef SOUND_THIN_API
+		POKEYSND_Init(POKEYSND_FREQ_17_EXACT, Sound_out.freq, Sound_out.channels, Sound_out.sample_size == 2 ? POKEYSND_BIT16 : 0);
+#elif defined(SUPPORTS_SOUND_REINIT)
 		Sound_Reinit();
-#endif
+#endif /* defined(SUPPORTS_SOUND_REINIT) */
+#endif /* SOUND */
 #if defined(DIRECTX)
 		SetTVModeMenuItem(mode);
 #endif
