@@ -95,24 +95,25 @@ typedef struct {
 	UWORD addr;
 } symtable_rec;
 
- /* Symbol names taken from atari.equ - part of disassembler by Erich Bacher
-    and from antic.h, gtia.h, pia.h and pokey.h.
+ /* Symbol names taken from:
+    - XL OS sources, Atari Home Computer System - Operating System Manual - XL Addendum
+    - BASIC sources, The Atari BASIC Source Book
     Symbols must be sorted by address. If the address has different names
     when reading/writing to it, put the read name first. */
 
 static const symtable_rec symtable_builtin[] = {
-	{"NGFLAG",  0x0001}, {"CASINI",  0x0002}, {"CASINI+1",0x0003}, {"RAMLO",   0x0004},
-	{"RAMLO+1", 0x0005}, {"TRAMSZ",  0x0006}, {"CMCMD",   0x0007}, {"WARMST",  0x0008},
-	{"BOOT",    0x0009}, {"DOSVEC",  0x000a}, {"DOSVEC+1",0x000b}, {"DOSINI",  0x000c},
-	{"DOSINI+1",0x000d}, {"APPMHI",  0x000e}, {"APPMHI+1",0x000f}, {"POKMSK",  0x0010},
-	{"BRKKEY",  0x0011}, {"RTCLOK",  0x0012}, {"RTCLOK+1",0x0013}, {"RTCLOK+2",0x0014},
-	{"BUFADR",  0x0015}, {"BUFADR+1",0x0016}, {"ICCOMT",  0x0017}, {"DSKFMS",  0x0018},
-	{"DSKFMS+1",0x0019}, {"DSKUTL",  0x001a}, {"DSKUTL+1",0x001b}, {"ABUFPT",  0x001c},
-	{"ABUFPT+1",0x001d}, {"ABUFPT+2",0x001e}, {"ABUFPT+3",0x001f},
+	{"LNFLG",   0x0000}, {"NGFLAG",  0x0001}, {"CASINI",  0x0002}, {"CASINI+1",0x0003},
+	{"RAMLO",   0x0004}, {"RAMLO+1", 0x0005}, {"TRAMSZ",  0x0006}, {"CMCMD",   0x0007},
+	{"WARMST",  0x0008}, {"BOOT?",   0x0009}, {"DOSVEC",  0x000a}, {"DOSVEC+1",0x000b},
+	{"DOSINI",  0x000c}, {"DOSINI+1",0x000d}, {"APPMHI",  0x000e}, {"APPMHI+1",0x000f},
+	{"POKMSK",  0x0010}, {"BRKKEY",  0x0011}, {"RTCLOK",  0x0012}, {"RTCLOK+1",0x0013},
+	{"RTCLOK+2",0x0014}, {"BUFADR",  0x0015}, {"BUFADR+1",0x0016}, {"ICCOMT",  0x0017},
+	{"DSKFMS",  0x0018}, {"DSKFMS+1",0x0019}, {"DSKUTL",  0x001a}, {"DSKUTL+1",0x001b},
+	{"ABUFPT",  0x001c}, {"ABUFPT+1",0x001d}, {"ABUFPT+2",0x001e}, {"ABUFPT+3",0x001f},
 	{"ICHIDZ",  0x0020}, {"ICDNOZ",  0x0021}, {"ICCOMZ",  0x0022}, {"ICSTAZ",  0x0023},
 	{"ICBALZ",  0x0024}, {"ICBAHZ",  0x0025}, {"ICPTLZ",  0x0026}, {"ICPTHZ",  0x0027},
 	{"ICBLLZ",  0x0028}, {"ICBLHZ",  0x0029}, {"ICAX1Z",  0x002a}, {"ICAX2Z",  0x002b},
-	{"ICAX3Z",  0x002c}, {"ICAX4Z",  0x002d}, {"ICAX5Z",  0x002e}, {"ICAX6Z",  0x002f},
+	{"ICSPRZ",  0x002c}, {"ICSPRZ+1",0x002d}, {"ICIDNO",  0x002e}, {"CIOCHR",  0x002f},
 	{"STATUS",  0x0030}, {"CHKSUM",  0x0031}, {"BUFRLO",  0x0032}, {"BUFRHI",  0x0033},
 	{"BFENLO",  0x0034}, {"BFENHI",  0x0035}, {"LTEMP",   0x0036}, {"LTEMP+1", 0x0037},
 	{"BUFRFL",  0x0038}, {"RECVDN",  0x0039}, {"XMTDON",  0x003a}, {"CHKSNT",  0x003b},
@@ -121,38 +122,50 @@ static const symtable_rec symtable_builtin[] = {
 	{"FMSZPG+1",0x0044}, {"FMSZPG+2",0x0045}, {"FMSZPG+3",0x0046}, {"FMSZPG+4",0x0047},
 	{"FMSZPG+5",0x0048}, {"FMSZPG+6",0x0049}, {"ZCHAIN",  0x004a}, {"ZCHAIN+1",0x004b},
 	{"DSTAT",   0x004c}, {"ATRACT",  0x004d}, {"DRKMSK",  0x004e}, {"COLRSH",  0x004f},
-	{"TEMP",    0x0050}, {"HOLD1",   0x0051}, {"LMARGN",  0x0052}, {"RMARGN",  0x0053},
+	{"TMPCHR",  0x0050}, {"HOLD1",   0x0051}, {"LMARGN",  0x0052}, {"RMARGN",  0x0053},
 	{"ROWCRS",  0x0054}, {"COLCRS",  0x0055}, {"COLCRS+1",0x0056}, {"DINDEX",  0x0057},
 	{"SAVMSC",  0x0058}, {"SAVMSC+1",0x0059}, {"OLDROW",  0x005a}, {"OLDCOL",  0x005b},
 	{"OLDCOL+1",0x005c}, {"OLDCHR",  0x005d}, {"OLDADR",  0x005e}, {"OLDADR+1",0x005f},
 	{"FKDEF",   0x0060}, {"FKDEF+1", 0x0061}, {"PALNTS",  0x0062}, {"LOGCOL",  0x0063},
-	{"ADRESS",  0x0064}, {"ADRESS+1",0x0065}, {"MLTTMP",  0x0066}, {"MLTTMP+1",0x0067},
+	{"ADRESS",  0x0064}, {"ADRESS+1",0x0065}, {"TOADR",   0x0066}, {"TOADR+1", 0x0067},
 	{"SAVADR",  0x0068}, {"SAVADR+1",0x0069}, {"RAMTOP",  0x006a}, {"BUFCNT",  0x006b},
 	{"BUFSTR",  0x006c}, {"BUFSTR+1",0x006d}, {"BITMSK",  0x006e}, {"SHFAMT",  0x006f},
 	{"ROWAC",   0x0070}, {"ROWAC+1", 0x0071}, {"COLAC",   0x0072}, {"COLAC+1", 0x0073},
 	{"ENDPT",   0x0074}, {"ENDPT+1", 0x0075}, {"DELTAR",  0x0076}, {"DELTAC",  0x0077},
 	{"DELTAC+1",0x0078}, {"KEYDEF",  0x0079}, {"KEYDEF+1",0x007a}, {"SWPFLG",  0x007b},
 	{"HOLDCH",  0x007c}, {"INSDAT",  0x007d}, {"COUNTR",  0x007e}, {"COUNTR+1",0x007f},
+
 	{"LOMEM",   0x0080}, {"LOMEM+1", 0x0081}, {"VNTP",    0x0082}, {"VNTP+1",  0x0083},
 	{"VNTD",    0x0084}, {"VNTD+1",  0x0085}, {"VVTP",    0x0086}, {"VVTP+1",  0x0087},
 	{"STMTAB",  0x0088}, {"STMTAB+1",0x0089}, {"STMCUR",  0x008a}, {"STMCUR+1",0x008b},
 	{"STARP",   0x008c}, {"STARP+1", 0x008d}, {"RUNSTK",  0x008e}, {"RUNSTK+1",0x008f},
-	{"TOPSTK",  0x0090}, {"TOPSTK+1",0x0091}, {"MEOLFLG", 0x0092}, {"POKADR",  0x0095},
-	{"POKADR+1",0x0096}, {"DATAD",   0x00b6}, {"DATALN",  0x00b7}, {"DATALN+1",0x00b8},
-	{"STOPLN",  0x00ba}, {"STOPLN+1",0x00bb}, {"SAVCUR",  0x00be}, {"IOCMD",   0x00c0},
-	{"IODVC",   0x00c1}, {"PROMPT",  0x00c2}, {"ERRSAVE", 0x00c3}, {"COLOR",  0x00c8},
-	{"PTABW",   0x00c9}, {"LOADFLG", 0x00ca}, {"FR0",     0x00d4}, {"FR0+1",   0x00d5},
-	{"FR0+2",   0x00d6}, {"FR0+3",   0x00d7}, {"FR0+4",   0x00d8}, {"FR0+5",   0x00d9},
-	{"FRE",     0x00da}, {"FRE+1",   0x00db}, {"FRE+2",   0x00dc}, {"FRE+3",   0x00dd},
-	{"FRE+4",   0x00de}, {"FRE+5",   0x00df}, {"FR1",     0x00e0}, {"FR1+1",   0x00e1},
-	{"FR1+2",   0x00e2}, {"FR1+3",   0x00e3}, {"FR1+4",   0x00e4}, {"FR1+5",   0x00e5},
-	{"FR2",     0x00e6}, {"FR2+1",   0x00e7}, {"FR2+2",   0x00e8}, {"FR2+3",   0x00e9},
-	{"FR2+4",   0x00ea}, {"FR2+5",   0x00eb}, {"FRX",     0x00ec}, {"EEXP",    0x00ed},
-	{"NSIGN",   0x00ee}, {"ESIGN",   0x00ef}, {"FCHRFLG", 0x00f0}, {"DIGRT",   0x00f1},
-	{"CIX",     0x00f2}, {"INBUFF",  0x00f3}, {"INBUFF+1",0x00f4}, {"ZTEMP1",  0x00f5},
-	{"ZTEMP1+1",0x00f6}, {"ZTEMP4",  0x00f7}, {"ZTEMP4+1",0x00f8}, {"ZTEMP3",  0x00f9},
-	{"ZTEMP3+1",0x00fa}, {"RADFLG",  0x00fb}, {"FLPTR",   0x00fc}, {"FLPTR+1", 0x00fd},
-	{"FPTR2",   0x00fe}, {"FPTR2+1", 0x00ff},
+	{"MEMTOP",  0x0090}, {"MEMTOP+1",0x0091}, {"MEOLFLG", 0x0092},
+	{"COX",     0x0094}, {"POKADR",  0x0095}, {"POKADR+1",0x0096}, {"SVESA",   0x0097},
+	{"SVESA+1", 0x0098}, {"MVFA",    0x0099}, {"MVFA+1",  0x009a}, {"MVTA",    0x009b},
+	{"MVTA+1",  0x009c}, {"CPC",     0x009d}, {"CPC+1",   0x009e}, {"LLNGTH",  0x009f},
+	{"TSLNUM",  0x00a0}, {"TSLNUM+1",0x00a1}, {"MVLNG",   0x00a2}, {"MVLNG+1", 0x00a3},
+	{"ECSIZE",  0x00a4}, {"ECSIZE+1",0x00a5}, {"DIRFLG",  0x00a6}, {"STMLBD",  0x00a7},
+	{"STINDEX", 0x00a8}, {"OPSTKX",  0x00a9}, {"ARSTKX",  0x00aa}, {"EXSVOP",  0x00ab},
+	{"EXSVPR",  0x00ac}, {"LELNUM",  0x00ad}, {"LELNUM+1",0x00ae}, {"STENUM",  0x00af},
+	{"COMCNT",  0x00b0}, {"ADFLAG",  0x00b1}, {"SVDISP",  0x00b2}, {"ONLOOP",  0x00b3},
+	{"ENTDTD",  0x00b4}, {"LISTDTD", 0x00b5}, {"DATAD",   0x00b6}, {"DATALN",  0x00b7},
+	{"DATALN+1",0x00b8}, {"ERRNUM",  0x00b9}, {"STOPLN",  0x00ba}, {"STOPLN+1",0x00bb},
+	{"TRAPLN",  0x00bc}, {"TRAPLN+1",0x00bd}, {"SAVCUR",  0x00be}, {"SAVCUR+1",0x00bf},
+	{"IOCMD",   0x00c0}, {"IODVC",   0x00c1}, {"PROMPT",  0x00c2}, {"ERRSAV",  0x00c3},
+	{"TEMPA",   0x00c4}, {"TEMPA+1", 0x00c5}, {"ZTEMP2",  0x00c6}, {"ZTEMP2+1",0x00c7},
+	{"COLOR",   0x00c8}, {"PTABW",   0x00c9}, {"LOADFLG", 0x00ca},
+	{"VTYPE",   0x00d2}, {"VNUM",    0x00d3},
+	{"FR0",     0x00d4}, {"FR0+1",   0x00d5}, {"FR0+2",   0x00d6}, {"FR0+3",   0x00d7},
+	{"FR0+4",   0x00d8}, {"FR0+5",   0x00d9}, {"FRE",     0x00da}, {"FRE+1",   0x00db},
+	{"FRE+2",   0x00dc}, {"FRE+3",   0x00dd}, {"FRE+4",   0x00de}, {"FRE+5",   0x00df},
+	{"FR1",     0x00e0}, {"FR1+1",   0x00e1}, {"FR1+2",   0x00e2}, {"FR1+3",   0x00e3},
+	{"FR1+4",   0x00e4}, {"FR1+5",   0x00e5}, {"FR2",     0x00e6}, {"FR2+1",   0x00e7},
+	{"FR2+2",   0x00e8}, {"FR2+3",   0x00e9}, {"FR2+4",   0x00ea}, {"FR2+5",   0x00eb},
+	{"FRX",     0x00ec}, {"EEXP",    0x00ed}, {"NSIGN",   0x00ee}, {"ESIGN",   0x00ef},
+	{"FCHRFLG", 0x00f0}, {"DIGRT",   0x00f1}, {"CIX",     0x00f2}, {"INBUFF",  0x00f3},
+	{"INBUFF+1",0x00f4}, {"ZTEMP1",  0x00f5}, {"ZTEMP1+1",0x00f6}, {"ZTEMP4",  0x00f7},
+	{"ZTEMP4+1",0x00f8}, {"ZTEMP3",  0x00f9}, {"ZTEMP3+1",0x00fa}, {"RADFLG",  0x00fb},
+	{"FLPTR",   0x00fc}, {"FLPTR+1", 0x00fd}, {"FPTR2",   0x00fe}, {"FPTR2+1", 0x00ff},
 
 	{"VDSLST",  0x0200}, {"VDSLST+1",0x0201}, {"VPRCED",  0x0202}, {"VPRCED+1",0x0203},
 	{"VINTER",  0x0204}, {"VINTER+1",0x0205}, {"VBREAK",  0x0206}, {"VBREAK+1",0x0207},
@@ -166,60 +179,60 @@ static const symtable_rec symtable_builtin[] = {
 	{"VVBLKD",  0x0224}, {"VVBLKD+1",0x0225}, {"CDTMA1",  0x0226}, {"CDTMA1+1",0x0227},
 	{"CDTMA2",  0x0228}, {"CDTMA2+1",0x0229}, {"CDTMF3",  0x022a}, {"SRTIMR",  0x022b},
 	{"CDTMF4",  0x022c}, {"INTEMP",  0x022d}, {"CDTMF5",  0x022e}, {"SDMCTL",  0x022f},
-	{"SDLSTL",  0x0230}, {"SDLSTH",  0x0231}, {"SSKCTL",  0x0232}, {"SPARE",   0x0233},
+	{"SDLSTL",  0x0230}, {"SDLSTH",  0x0231}, {"SSKCTL",  0x0232}, {"LCOUNT",  0x0233},
 	{"LPENH",   0x0234}, {"LPENV",   0x0235}, {"BRKKY",   0x0236}, {"BRKKY+1", 0x0237},
 	{"VPIRQ",   0x0238}, {"VPIRQ+1", 0x0239}, {"CDEVIC",  0x023a}, {"CCOMND",  0x023b},
-	{"CAUX1",   0x023c}, {"CAUX2",   0x023d}, {"TMPSIO",  0x023e}, {"ERRFLG",  0x023f},
+	{"CAUX1",   0x023c}, {"CAUX2",   0x023d}, {"TEMP",    0x023e}, {"ERRFLG",  0x023f},
 	{"DFLAGS",  0x0240}, {"DBSECT",  0x0241}, {"BOOTAD",  0x0242}, {"BOOTAD+1",0x0243},
 	{"COLDST",  0x0244}, {"RECLEN",  0x0245}, {"DSKTIM",  0x0246}, {"PDVMSK",  0x0247},
-	{"SHPDVS",  0x0248}, {"PDMSK",   0x0249}, {"RELADR",  0x024a}, {"RELADR+1",0x024b},
-	{"PPTMPA",  0x024c}, {"PPTMPX",  0x024d}, {"CHSALT",  0x026b}, {"VSFLAG",  0x026c},
-	{"KEYDIS",  0x026d}, {"FINE",    0x026e}, {"GPRIOR",  0x026f}, {"PADDL0",  0x0270},
-	{"PADDL1",  0x0271}, {"PADDL2",  0x0272}, {"PADDL3",  0x0273}, {"PADDL4",  0x0274},
-	{"PADDL5",  0x0275}, {"PADDL6",  0x0276}, {"PADDL7",  0x0277}, {"STICK0",  0x0278},
-	{"STICK1",  0x0279}, {"STICK2",  0x027a}, {"STICK3",  0x027b}, {"PTRIG0",  0x027c},
-	{"PTRIG1",  0x027d}, {"PTRIG2",  0x027e}, {"PTRIG3",  0x027f}, {"PTRIG4",  0x0280},
-	{"PTRIG5",  0x0281}, {"PTRIG6",  0x0282}, {"PTRIG7",  0x0283}, {"STRIG0",  0x0284},
-	{"STRIG1",  0x0285}, {"STRIG2",  0x0286}, {"STRIG3",  0x0287}, {"HIBYTE",  0x0288},
-	{"WMODE",   0x0289}, {"BLIM",    0x028a}, {"IMASK",   0x028b}, {"JVECK",   0x028c},
-	{"NEWADR",  0x028e}, {"TXTROW",  0x0290}, {"TXTCOL",  0x0291}, {"TXTCOL+1",0x0292},
-	{"TINDEX",  0x0293}, {"TXTMSC",  0x0294}, {"TXTMSC+1",0x0295}, {"TXTOLD",  0x0296},
-	{"TXTOLD+1",0x0297}, {"TXTOLD+2",0x0298}, {"TXTOLD+3",0x0299}, {"TXTOLD+4",0x029a},
-	{"TXTOLD+5",0x029b}, {"CRETRY",  0x029c}, {"HOLD3",   0x029d}, {"SUBTMP",  0x029e},
-	{"HOLD2",   0x029f}, {"DMASK",   0x02a0}, {"TMPLBT",  0x02a1}, {"ESCFLG",  0x02a2},
-	{"TABMAP",  0x02a3}, {"TABMAP+1",0x02a4}, {"TABMAP+2",0x02a5}, {"TABMAP+3",0x02a6},
-	{"TABMAP+4",0x02a7}, {"TABMAP+5",0x02a8}, {"TABMAP+6",0x02a9}, {"TABMAP+7",0x02aa},
-	{"TABMAP+8",0x02ab}, {"TABMAP+9",0x02ac}, {"TABMAP+A",0x02ad}, {"TABMAP+B",0x02ae},
-	{"TABMAP+C",0x02af}, {"TABMAP+D",0x02b0}, {"TABMAP+E",0x02b1}, {"LOGMAP",  0x02b2},
-	{"LOGMAP+1",0x02b3}, {"LOGMAP+2",0x02b4}, {"LOGMAP+3",0x02b5}, {"INVFLG",  0x02b6},
-	{"FILFLG",  0x02b7}, {"TMPROW",  0x02b8}, {"TMPCOL",  0x02b9}, {"TMPCOL+1",0x02ba},
-	{"SCRFLG",  0x02bb}, {"HOLD4",   0x02bc}, {"DRETRY",  0x02bd}, {"SHFLOC",  0x02be},
-	{"BOTSCR",  0x02bf}, {"PCOLR0",  0x02c0}, {"PCOLR1",  0x02c1}, {"PCOLR2",  0x02c2},
-	{"PCOLR3",  0x02c3}, {"COLOR0",  0x02c4}, {"COLOR1",  0x02c5}, {"COLOR2",  0x02c6},
-	{"COLOR3",  0x02c7}, {"COLOR4",  0x02c8}, {"RUNADR",  0x02c9}, {"RUNADR+1",0x02ca},
-	{"HIUSED",  0x02cb}, {"HIUSED+1",0x02cc}, {"ZHIUSE",  0x02cd}, {"ZHIUSE+1",0x02ce},
-	{"GBYTEA",  0x02cf}, {"GBYTEA+1",0x02d0}, {"LOADAD",  0x02d1}, {"LOADAD+1",0x02d2},
-	{"ZLOADA",  0x02d3}, {"ZLOADA+1",0x02d4}, {"DSCTLN",  0x02d5}, {"DSCTLN+1",0x02d6},
-	{"ACMISR",  0x02d7}, {"ACMISR+1",0x02d8}, {"KRPDER",  0x02d9}, {"KEYREP",  0x02da},
-	{"NOCLIK",  0x02db}, {"HELPFG",  0x02dc}, {"DMASAV",  0x02dd}, {"PBPNT",   0x02de},
-	{"PBUFSZ",  0x02df}, {"RUNAD",   0x02e0}, {"RUNAD+1", 0x02e1}, {"INITAD",  0x02e2},
-	{"INITAD+1",0x02e3}, {"RAMSIZ",  0x02e4}, {"MEMTOP",  0x02e5}, {"MEMTOP+1",0x02e6},
-	{"MEMLO",   0x02e7}, {"MEMLO+1", 0x02e8}, {"HNDLOD",  0x02e9}, {"DVSTAT",  0x02ea},
-	{"DVSTAT+1",0x02eb}, {"DVSTAT+2",0x02ec}, {"DVSTAT+3",0x02ed}, {"CBAUDL",  0x02ee},
-	{"CBAUDH",  0x02ef}, {"CRSINH",  0x02f0}, {"KEYDEL",  0x02f1}, {"CH1",     0x02f2},
-	{"CHACT",   0x02f3}, {"CHBAS",   0x02f4}, {"NEWROW",  0x02f5}, {"NEWCOL",  0x02f6},
-	{"NEWCOL+1",0x02f7}, {"ROWINC",  0x02f8}, {"COLINC",  0x02f9}, {"CHAR",    0x02fa},
-	{"ATACHR",  0x02fb}, {"CH",      0x02fc}, {"FILDAT",  0x02fd}, {"DSPFLG",  0x02fe},
-	{"SSFLAG",  0x02ff},
+	{"SHPDVS",  0x0248}, {"PDIMSK",  0x0249}, {"RELADR",  0x024a}, {"RELADR+1",0x024b},
+	{"PPTMPA",  0x024c}, {"PPTMPX",  0x024d}, {"CHSALT",  0x026b},
+	{"VSFLAG",  0x026c}, {"KEYDIS",  0x026d}, {"FINE",    0x026e}, {"GPRIOR",  0x026f},
+	{"PADDL0",  0x0270}, {"PADDL1",  0x0271}, {"PADDL2",  0x0272}, {"PADDL3",  0x0273},
+	{"PADDL4",  0x0274}, {"PADDL5",  0x0275}, {"PADDL6",  0x0276}, {"PADDL7",  0x0277},
+	{"STICK0",  0x0278}, {"STICK1",  0x0279}, {"STICK2",  0x027a}, {"STICK3",  0x027b},
+	{"PTRIG0",  0x027c}, {"PTRIG1",  0x027d}, {"PTRIG2",  0x027e}, {"PTRIG3",  0x027f},
+	{"PTRIG4",  0x0280}, {"PTRIG5",  0x0281}, {"PTRIG6",  0x0282}, {"PTRIG7",  0x0283},
+	{"STRIG0",  0x0284}, {"STRIG1",  0x0285}, {"STRIG2",  0x0286}, {"STRIG3",  0x0287},
+	{"HIBYTE",  0x0288}, {"WMODE",   0x0289}, {"BLIM",    0x028a}, {"IMASK",   0x028b},
+	{"JVECK",   0x028c}, {"JVECK+1", 0x028d}, {"NEWADR",  0x028e}, {"NEWADR+1",0x028f},
+	{"TXTROW",  0x0290}, {"TXTCOL",  0x0291}, {"TXTCOL+1",0x0292}, {"TINDEX",  0x0293},
+	{"TXTMSC",  0x0294}, {"TXTMSC+1",0x0295}, {"TXTOLD",  0x0296}, {"TXTOLD+1",0x0297},
+	{"TXTOLD+2",0x0298}, {"TXTOLD+3",0x0299}, {"TXTOLD+4",0x029a}, {"TXTOLD+5",0x029b},
+	{"CRETRY",  0x029c}, {"HOLD3",   0x029d}, {"SUBTMP",  0x029e}, {"HOLD2",   0x029f},
+	{"DMASK",   0x02a0}, {"TMPLBT",  0x02a1}, {"ESCFLG",  0x02a2}, {"TABMAP",  0x02a3},
+	{"TABMAP+1",0x02a4}, {"TABMAP+2",0x02a5}, {"TABMAP+3",0x02a6}, {"TABMAP+4",0x02a7},
+	{"TABMAP+5",0x02a8}, {"TABMAP+6",0x02a9}, {"TABMAP+7",0x02aa}, {"TABMAP+8",0x02ab},
+	{"TABMAP+9",0x02ac}, {"TABMAP+A",0x02ad}, {"TABMAP+B",0x02ae}, {"TABMAP+C",0x02af},
+	{"TABMAP+D",0x02b0}, {"TABMAP+E",0x02b1}, {"LOGMAP",  0x02b2}, {"LOGMAP+1",0x02b3},
+	{"LOGMAP+2",0x02b4}, {"LOGMAP+3",0x02b5}, {"INVFLG",  0x02b6}, {"FILFLG",  0x02b7},
+	{"TMPROW",  0x02b8}, {"TMPCOL",  0x02b9}, {"TMPCOL+1",0x02ba}, {"SCRFLG",  0x02bb},
+	{"HOLD4",   0x02bc}, {"DRETRY",  0x02bd}, {"SHFLOK",  0x02be}, {"BOTSCR",  0x02bf},
+	{"PCOLR0",  0x02c0}, {"PCOLR1",  0x02c1}, {"PCOLR2",  0x02c2}, {"PCOLR3",  0x02c3},
+	{"COLOR0",  0x02c4}, {"COLOR1",  0x02c5}, {"COLOR2",  0x02c6}, {"COLOR3",  0x02c7},
+	{"COLOR4",  0x02c8}, {"RUNADR",  0x02c9}, {"RUNADR+1",0x02ca}, {"HIUSED",  0x02cb},
+	{"HIUSED+1",0x02cc}, {"ZHIUSE",  0x02cd}, {"ZHIUSE+1",0x02ce}, {"GBYTEA",  0x02cf},
+	{"GBYTEA+1",0x02d0}, {"LOADAD",  0x02d1}, {"LOADAD+1",0x02d2}, {"ZLOADA",  0x02d3},
+	{"ZLOADA+1",0x02d4}, {"DSCTLN",  0x02d5}, {"DSCTLN+1",0x02d6}, {"ACMISR",  0x02d7},
+	{"ACMISR+1",0x02d8}, {"KRPDEL",  0x02d9}, {"KEYREP",  0x02da}, {"NOCLIK",  0x02db},
+	{"HELPFG",  0x02dc}, {"DMASAV",  0x02dd}, {"PBPNT",   0x02de}, {"PBUFSZ",  0x02df},
+	{"RUNAD",   0x02e0}, {"RUNAD+1", 0x02e1}, {"INITAD",  0x02e2}, {"INITAD+1",0x02e3},
+	{"RAMSIZ",  0x02e4}, {"MEMTOP",  0x02e5}, {"MEMTOP+1",0x02e6}, {"MEMLO",   0x02e7},
+	{"MEMLO+1", 0x02e8}, {"HNDLOD",  0x02e9}, {"DVSTAT",  0x02ea}, {"DVSTAT+1",0x02eb},
+	{"DVSTAT+2",0x02ec}, {"DVSTAT+3",0x02ed}, {"CBAUDL",  0x02ee}, {"CBAUDH",  0x02ef},
+	{"CRSINH",  0x02f0}, {"KEYDEL",  0x02f1}, {"CH1",     0x02f2}, {"CHACT",   0x02f3},
+	{"CHBAS",   0x02f4}, {"NEWROW",  0x02f5}, {"NEWCOL",  0x02f6}, {"NEWCOL+1",0x02f7},
+	{"ROWINC",  0x02f8}, {"COLINC",  0x02f9}, {"CHAR",    0x02fa}, {"ATACHR",  0x02fb},
+	{"CH",      0x02fc}, {"FILDAT",  0x02fd}, {"DSPFLG",  0x02fe}, {"SSFLAG",  0x02ff},
 
 	{"DDEVIC",  0x0300}, {"DUNIT",   0x0301}, {"DCOMND",  0x0302}, {"DSTATS",  0x0303},
 	{"DBUFLO",  0x0304}, {"DBUFHI",  0x0305}, {"DTIMLO",  0x0306}, {"DUNUSE",  0x0307},
 	{"DBYTLO",  0x0308}, {"DBYTHI",  0x0309}, {"DAUX1",   0x030a}, {"DAUX2",   0x030b},
 	{"TIMER1",  0x030c}, {"TIMER1+1",0x030d}, {"ADDCOR",  0x030e}, {"CASFLG",  0x030f},
-	{"TIMER2",  0x0310}, {"TIMER2+1",0x0311}, {"TEMP1",   0x0312}, {"TEMP1+1", 0x0313},
-	{"TEMP2",   0x0314}, {"TEMP3",   0x0315}, {"SAVIO",   0x0316}, {"TIMFLG",  0x0317},
+	{"TIMER2",  0x0310}, {"TIMER2+1",0x0311}, {"TEMP1",   0x0312}, {"TEMP2",   0x0313},
+	{"PTIMOT",  0x0314}, {"TEMP3",   0x0315}, {"SAVIO",   0x0316}, {"TIMFLG",  0x0317},
 	{"STACKP",  0x0318}, {"TSTAT",   0x0319}, {"HATABS",  0x031a}, /* HATABS 1-34 */
-	{"PUTBT1",  0x033d}, {"PUTBT2",  0x033e}, {"PUTBT3",  0x033f},
+	{"PUPBT1",  0x033d}, {"PUPBT2",  0x033e}, {"PUPBT3",  0x033f},
 	{"B0-ICHID",0x0340}, {"B0-ICDNO",0x0341}, {"B0-ICCOM",0x0342}, {"B0-ICSTA",0x0343},
 	{"B0-ICBAL",0x0344}, {"B0-ICBAH",0x0345}, {"B0-ICPTL",0x0346}, {"B0-ICPTH",0x0347},
 	{"B0-ICBLL",0x0348}, {"B0-ICBLH",0x0349}, {"B0-ICAX1",0x034a}, {"B0-ICAX2",0x034b},
@@ -258,6 +271,12 @@ static const symtable_rec symtable_builtin[] = {
 	{"BASICF",  0x03f8}, {"MINTLK",  0x03f9}, {"GINTLK",  0x03fa}, {"CHLINK",  0x03fb},
 	{"CHLINK+1",0x03fc}, {"CASBUF",  0x03fd},
 
+	{"R-CARTCS",0x9ffa}, {"R-CARTCS+1",0x9ffb},
+	{"R-CART",  0x9ffc}, {"R-CARTFG",  0x9ffd}, {"R-CARTAD",0x9ffe}, {"R-CARTAD+1",0x9fff},
+
+	{"CARTCS",  0xbffa}, {"CARTCS+1",0xbffb},
+	{"CART",    0xbffc}, {"CARTFG",  0xbffd}, {"CARTAD",  0xbffe}, {"CARTAD+1",0xbfff},
+
 	{"M0PF",  0xd000}, {"HPOSP0",0xd000}, {"M1PF",  0xd001}, {"HPOSP1",0xd001},
 	{"M2PF",  0xd002}, {"HPOSP2",0xd002}, {"M3PF",  0xd003}, {"HPOSP3",0xd003},
 	{"P0PF",  0xd004}, {"HPOSM0",0xd004}, {"P1PF",  0xd005}, {"HPOSM1",0xd005},
@@ -273,12 +292,14 @@ static const symtable_rec symtable_builtin[] = {
 	{"COLPF2",0xd018}, {"COLPF3",0xd019}, {"COLBK", 0xd01a}, {"PRIOR", 0xd01b},
 	{"VDELAY",0xd01c}, {"GRACTL",0xd01d}, {"HITCLR",0xd01e}, {"CONSOL",0xd01f},
 
+	{"PBI",   0xd100}, /* PBI 1-254 */    {"PDVI",  0xd1ff}, {"PDVS",  0xd1ff},
+
 	{"POT0",  0xd200}, {"AUDF1", 0xd200}, {"POT1",  0xd201}, {"AUDC1", 0xd201},
 	{"POT2",  0xd202}, {"AUDF2", 0xd202}, {"POT3",  0xd203}, {"AUDC2", 0xd203},
 	{"POT4",  0xd204}, {"AUDF3", 0xd204}, {"POT5",  0xd205}, {"AUDC3", 0xd205},
 	{"POT6",  0xd206}, {"AUDF4", 0xd206}, {"POT7",  0xd207}, {"AUDC4", 0xd207},
 	{"ALLPOT",0xd208}, {"AUDCTL",0xd208}, {"KBCODE",0xd209}, {"STIMER",0xd209},
-	{"RANDOM",0xd20a}, {"SKREST",0xd20a}, {"POTGO", 0xd20b},
+	{"RANDOM",0xd20a}, {"SKRES", 0xd20a}, {"POTGO", 0xd20b},
 	{"SERIN", 0xd20d}, {"SEROUT",0xd20d}, {"IRQST", 0xd20e}, {"IRQEN", 0xd20e},
 	{"SKSTAT",0xd20f}, {"SKCTL", 0xd20f},
 
@@ -289,19 +310,29 @@ static const symtable_rec symtable_builtin[] = {
 	{"WSYNC", 0xd40a}, {"VCOUNT",0xd40b}, {"PENH",  0xd40c}, {"PENV",  0xd40d},
 	{"NMIEN", 0xd40e}, {"NMIST", 0xd40f}, {"NMIRES",0xd40f},
 
-	{"AFP",   0xd800}, {"FASC",  0xd8e6}, {"IFP",   0xd9aa}, {"FPI",   0xd9d2},
-	{"ZPRO",  0xda44}, {"ZF1",   0xda46}, {"FSUB",  0xda60}, {"FADD",  0xda66},
+	{"PBIRAM",0xd600}, /* PMIRAM 1-255 */
+
+	{"AFP",   0xd800},
+
+	{"PDID1",   0xd803}, {"PDIOV", 0xd805}, {"PDIOV+1",0xd806},{"PDIRQV", 0xd808},
+	{"PDIRQV+1",0xd809}, {"PDID2", 0xd80b}, {"PDVV", 0xd80d},
+
+	{"FASC",  0xd8e6}, {"IFP",   0xd9aa}, {"FPI",   0xd9d2},
+	{"ZFR0",  0xda44}, {"ZF1",   0xda46}, {"FSUB",  0xda60}, {"FADD",  0xda66},
 	{"FMUL",  0xdadb}, {"FDIV",  0xdb28}, {"PLYEVL",0xdd40}, {"FLD0R", 0xdd89},
-	{"FLD0R", 0xdd8d}, {"FLD1R", 0xdd98}, {"FLD1P", 0xdd9c}, {"FST0R", 0xdda7},
+	{"FLD0P", 0xdd8d}, {"FLD1R", 0xdd98}, {"FLD1P", 0xdd9c}, {"FST0R", 0xdda7},
 	{"FST0P", 0xddab}, {"FMOVE", 0xddb6}, {"EXP",   0xddc0}, {"EXP10", 0xddcc},
 	{"LOG",   0xdecd}, {"LOG10", 0xded1},
 
-	{"DSKINV",0xe453}, {"CIOV",  0xe456}, {"SIOV",  0xe459}, {"SETVBV",0xe45c},
-	{"SYSVBV",0xe45f}, {"XITVBV",0xe462}, {"SIOINV",0xe465}, {"SENDEV",0xe468},
-	{"INTINV",0xe46b}, {"CIOINV",0xe46e}, {"SELFSV",0xe471}, {"WARMSV",0xe474},
-	{"COLDSV",0xe477}, {"RBLOKV",0xe47a}, {"CSOPIV",0xe47d}, {"PUPDIV",0xe480},
-	{"SELFTSV",0xe483},{"PENTV", 0xe486}, {"PHUNLV",0xe489}, {"PHINIV",0xe48c},
-	{"GPDVV", 0xe48f},
+	{"EDITRV",0xe400}, {"SCRENV",0xe410}, {"KEYBDV",0xe420}, {"PRINTV",0xe430},
+	{"CASETV",0xe440},
+
+	{"DINITV",0xe450}, {"DSKINV",0xe453}, {"CIOV",  0xe456}, {"SIOV",  0xe459},
+	{"SETVBV",0xe45c}, {"SYSVBV",0xe45f}, {"XITVBV",0xe462}, {"SIOINV",0xe465},
+	{"SENDEV",0xe468}, {"INTINV",0xe46b}, {"CIOINV",0xe46e}, {"BLKBDV",0xe471},
+	{"WARMSV",0xe474}, {"COLDSV",0xe477}, {"RBLOKV",0xe47a}, {"CSOPIV",0xe47d},
+	{"PUPDIV",0xe480}, {"SLFTSV",0xe483}, {"PHENTV",0xe486}, {"PHUNLV",0xe489},
+	{"PHINIV",0xe48c}, {"GPDVV", 0xe48f},
 
 	{NULL,    0x0000}
 };
@@ -347,7 +378,7 @@ static const symtable_rec symtable_builtin_5200[] = {
 	{"POT4",  0xe804}, {"AUDF3", 0xe804}, {"POT5",  0xe805}, {"AUDC3", 0xe805},
 	{"POT6",  0xe806}, {"AUDF4", 0xe806}, {"POT7",  0xe807}, {"AUDC4", 0xe807},
 	{"ALLPOT",0xe808}, {"AUDCTL",0xe808}, {"KBCODE",0xe809}, {"STIMER",0xe809},
-	{"RANDOM",0xe80a}, {"SKREST",0xe80a}, {"POTGO", 0xe80b},
+	{"RANDOM",0xe80a}, {"SKRES", 0xe80a}, {"POTGO", 0xe80b},
 	{"SERIN", 0xe80d}, {"SEROUT",0xe80d}, {"IRQST", 0xe80e}, {"IRQEN", 0xe80e},
 	{"SKSTAT",0xe80f}, {"SKCTL", 0xe80f},
 
