@@ -97,7 +97,7 @@
 #endif
 #endif
 
-#ifdef WIN32
+#ifdef HAVE_WINDOWS_H
 #ifdef R_SERIAL
 #error The Windows version does not support the serial R: device
 #endif
@@ -717,7 +717,7 @@ static void xio_40(void)
   else
   {
     concurrent = 0;
-    sprintf(MESSAGE, "R*: XIO 40, %d", aux1);
+    snprintf(MESSAGE, sizeof(MESSAGE), "R*: XIO 40, %d", aux1);
     DBG_APRINT(MESSAGE);
   }
 
@@ -730,7 +730,7 @@ static void xio_40(void)
 static void open_connection(char * address, int port)
 {
   struct hostent *host;
-#ifdef WIN32
+#ifdef HAVE_WINDOWS_H
   static int winsock_started;
   WSADATA wdata;
   WORD ver = MAKEWORD(1,1);
@@ -742,7 +742,7 @@ static void open_connection(char * address, int port)
     }
     winsock_started = 1;
   }
-#endif /* WIN32 */
+#endif /* HAVE_WINDOWS_H */
   if((address != NULL) && (strlen(address) > 0))
   {
     close(rdev_fd);
@@ -752,7 +752,7 @@ static void open_connection(char * address, int port)
     memset ( &peer_in, 0, sizeof ( struct sockaddr_in ) );
     /*rdev_fd = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);*/
     rdev_fd = socket(AF_INET, SOCK_STREAM, 0);
-#ifdef WIN32
+#ifdef HAVE_WINDOWS_H
     ioctlsocket(rdev_fd, FIONBIO, &ioctlsocket_non_block);
 #else
     fcntl(rdev_fd, F_SETFL, O_NONBLOCK);
@@ -765,7 +765,7 @@ static void open_connection(char * address, int port)
         host = gethostbyname(address);
         if(host != NULL)
         {
-          sprintf(MESSAGE, "R*: Host = '%s'.",  host->h_name);
+          snprintf(MESSAGE, sizeof(MESSAGE), "R*: Host = '%s'.",  host->h_name);
           DBG_APRINT(MESSAGE);
           memcpy((caddr_t)&peer_in.sin_addr, host->h_addr_list[0], host->h_length);
         }
@@ -787,24 +787,24 @@ static void open_connection(char * address, int port)
     if(connect(rdev_fd, (struct sockaddr *)&peer_in, sizeof(peer_in)) < 0)
     {
 #ifdef DEBUG
-      sprintf(MESSAGE, "R*: connect: '%s'", strerror(errno));
+      snprintf(MESSAGE, sizeof(MESSAGE), "R*: connect: '%s'", strerror(errno));
       DBG_APRINT(MESSAGE);
 #endif
     }
-#ifndef WIN32
+#ifndef HAVE_WINDOWS_H
     signal(SIGPIPE, catch_disconnect); /*Need to see if the other end disconnects...*/
     signal(SIGHUP, catch_disconnect); /*Need to see if the other end disconnects...*/
-#endif /* WIN32 */
-    sprintf(MESSAGE, "R*: Connecting to %s", address);
+#endif /* HAVE_WINDOWS_H */
+    snprintf(MESSAGE, sizeof(MESSAGE), "R*: Connecting to %s", address);
     DBG_APRINT(MESSAGE);
-#ifdef WIN32
+#ifdef HAVE_WINDOWS_H
     ioctlsocket(rdev_fd, FIONBIO, &ioctlsocket_non_block);
 #else
     fcntl(rdev_fd, F_SETFL, O_NONBLOCK);
-#endif /* WIN32 */
+#endif /* HAVE_WINDOWS_H */
 
     /* Telnet negotiation */
-    sprintf(MESSAGE, "%c%c%c%c%c%c%c%c%c", 0xff, 0xfb, 0x01, 0xff, 0xfb, 0x03, 0xff, 0xfd, 0x0f3);
+    snprintf(MESSAGE, sizeof(MESSAGE), "%c%c%c%c%c%c%c%c%c", 0xff, 0xfb, 0x01, 0xff, 0xfb, 0x03, 0xff, 0xfd, 0x0f3);
     write(rdev_fd, MESSAGE, 9);
     DBG_APRINT("R*: Negotiating Terminal Options...");
   }
@@ -850,7 +850,7 @@ static void open_connection_serial(int port)
 
   dev_name[strlen(dev_name) - 1] += port - 1;
 
-  sprintf(MESSAGE, "R*: using serial device %s", dev_name);
+  snprintf(MESSAGE, sizeof(MESSAGE), "R*: using serial device %s", dev_name);
   DBG_APRINT(MESSAGE);
 
   rdev_fd = open(dev_name, O_RDWR | O_NOCTTY | O_NDELAY);
@@ -1194,7 +1194,7 @@ void RDevice_WRIT(void)
 ---------------------------------------------------------------------------*/
 void RDevice_STAT(void)
 {
-#ifdef WIN32
+#ifdef HAVE_WINDOWS_H
   int len;
 #else
 #ifdef R_NETWORK
@@ -1240,14 +1240,14 @@ void RDevice_STAT(void)
         if(bind ( sock, (struct sockaddr *)&in, sizeof ( struct sockaddr_in ) ) < 0) perror("bind");
         listen ( sock, 5 );
         /* sethostent(1); */
-#ifdef WIN32
+#ifdef HAVE_WINDOWS_H
         retval = ioctlsocket(sock, FIONBIO, &ioctlsocket_non_block);
 #else
         retval = fcntl( sock, F_SETFL, O_NONBLOCK);
-#endif /* WIN32 */
+#endif /* HAVE_WINDOWS_H */
         len = sizeof ( struct sockaddr_in );
         /*bufend = 0;*/
-        sprintf(MESSAGE, "R%d: Listening on port %d...", devnum, portnum);
+        snprintf(MESSAGE, sizeof(MESSAGE), "R%d: Listening on port %d...", devnum, portnum);
         DBG_APRINT(MESSAGE);
       }
 
@@ -1261,28 +1261,28 @@ void RDevice_STAT(void)
         }
         else
         {
-          sprintf(MESSAGE, "R%d: Serving Connection from %s...", devnum, inet_ntoa(peer_in.sin_addr));
+          snprintf(MESSAGE, sizeof(MESSAGE), "R%d: Serving Connection from %s...", devnum, inet_ntoa(peer_in.sin_addr));
           if ((host = gethostbyaddr((char *) &peer_in.sin_addr, sizeof peer_in.sin_addr, AF_INET)) == NULL)
           {
             /*perror("gethostbyaddr");*/
             /*DBG_APRINT("Connected.");*/
           } else {
-            sprintf(MESSAGE, "R%d: Serving Connection from %s.", devnum, host->h_name);
+            snprintf(MESSAGE, sizeof(MESSAGE), "R%d: Serving Connection from %s.", devnum, host->h_name);
           }
         }
         DBG_APRINT(MESSAGE);
-#ifndef WIN32
+#ifndef HAVE_WINDOWS_H
         signal(SIGPIPE, catch_disconnect); /*Need to see if the other end disconnects...*/
         signal(SIGHUP, catch_disconnect); /*Need to see if the other end disconnects...*/
-#endif /* WIN32 */
-#ifdef WIN32
+#endif /* HAVE_WINDOWS_H */
+#ifdef HAVE_WINDOWS_H
         retval = ioctlsocket(rdev_fd, FIONBIO, &ioctlsocket_non_block);
 #else
         retval = fcntl( rdev_fd, F_SETFL, O_NONBLOCK);
-#endif /* WIN32 */
+#endif /* HAVE_WINDOWS_H */
 
         /* Telnet negotiation */
-        sprintf(MESSAGE, "%c%c%c%c%c%c%c%c%c", 0xff, 0xfb, 0x01, 0xff, 0xfb, 0x03, 0xff, 0xfd, 0x0f3);
+        snprintf(MESSAGE, sizeof(MESSAGE), "%c%c%c%c%c%c%c%c%c", 0xff, 0xfb, 0x01, 0xff, 0xfb, 0x03, 0xff, 0xfd, 0x0f3);
         write(rdev_fd, MESSAGE, 9);
         DBG_APRINT("R*: Negotiating Terminal Options...");
 
@@ -1399,7 +1399,7 @@ void RDevice_SPEC(void)
   int iccom;
 
   iccom = Peek(Devices_ICCOMZ);
-  sprintf(MESSAGE, "R*: XIO %d", iccom);
+  snprintf(MESSAGE, sizeof(MESSAGE), "R*: XIO %d", iccom);
   DBG_APRINT(MESSAGE);
 
 /*
@@ -1449,7 +1449,7 @@ void RDevice_INIT(void)
 
 void RDevice_Exit(void)
 {
-#ifdef WIN32
+#ifdef HAVE_WINDOWS_H
   WSACleanup();
-#endif /* WIN32 */
+#endif /* HAVE_WINDOWS_H */
 }
