@@ -2,7 +2,7 @@
  * sdl/main.c - SDL library specific port code - main interface
  *
  * Copyright (c) 2001-2002 Jacek Poplawski
- * Copyright (C) 2001-2009 Atari800 development team (see DOC/CREDITS)
+ * Copyright (C) 2001-2014 Atari800 development team (see DOC/CREDITS)
  *
  * This file is part of the Atari800 emulator project which emulates
  * the Atari 400, 800, 800XL, 130XE, and 5200 8-bit computers.
@@ -44,6 +44,10 @@
 #ifdef SOUND
 #include "../sound.h"
 #endif
+#ifdef USE_UI_BASIC_ONSCREEN_KEYBOARD
+#include "akey.h"
+#include "ui_basic.h"
+#endif
 #include "videomode.h"
 #include "sdl/video.h"
 #include "sdl/input.h"
@@ -99,43 +103,37 @@ int PLATFORM_Initialise(int *argc, char *argv[])
 
 int PLATFORM_Exit(int run_monitor)
 {
-	int restart;
-
 	SDL_INPUT_Exit();
 	/* If the SDL window was left not closed, it would be unusable and hanging
 	   for the time the monitor is active. Also, with SDL_VIDEODRIVER=directx all
 	   keyboard presses in console would be still fetched by the SDL window after
 	   leaving the monitor. To avoid the problems, close the video subsystem. */
 	SDL_VIDEO_Exit();
+	Log_flushlog();
+
 	if (run_monitor) {
 #ifdef SOUND
 		Sound_Pause();
 #endif
-		restart = MONITOR_Run();
-#ifdef SOUND
-		Sound_Continue();
+		if (MONITOR_Run()) {
+			/* Reinitialise the SDL subsystem. */
+#ifdef MONITOR_BREAK
+			if (!MONITOR_break_step) /*Do not initialise videomode when stepping through code */
 #endif
-	}
-	else {
-		restart = FALSE;
+			{
+				SDL_VIDEO_InitSDL();
+				SDL_INPUT_Restart();
+				/* This call reopens the SDL window. */
+				VIDEOMODE_Update();
+			}
+	#ifdef SOUND
+			Sound_Continue();
+	#endif
+			return 1;
+		}
 	}
 
-	if (restart) {
-		/* Reinitialise the SDL subsystem. */
-		SDL_VIDEO_InitSDL();
-		SDL_INPUT_Restart();
-		/* This call reopens the SDL window. */
-		VIDEOMODE_Update();
-		return 1;
-	}
-
-#ifdef SOUND
-	Sound_Exit();
-#endif
-
-	Log_flushlog();
-
-	return restart;
+	return 0;
 }
 
 #if HAVE_WINDOWS_H
@@ -177,6 +175,21 @@ int main(int argc, char **argv)
 	/* main loop */
 	for (;;) {
 		INPUT_key_code = PLATFORM_Keyboard();
+#ifdef USE_UI_BASIC_ONSCREEN_KEYBOARD
+		if (INPUT_key_code == AKEY_KEYB) {
+			Sound_Pause();
+			UI_BASIC_in_kbui = TRUE;
+			INPUT_key_code = UI_BASIC_OnScreenKeyboard(NULL, 0);
+			UI_BASIC_in_kbui = FALSE;
+			switch (INPUT_key_code) {
+				case AKEY_OPTION: INPUT_key_consol &= (~INPUT_CONSOL_OPTION); break;
+				case AKEY_SELECT: INPUT_key_consol &= (~INPUT_CONSOL_SELECT); break;
+				case AKEY_START: INPUT_key_consol &= (~INPUT_CONSOL_START); break;
+			}
+
+			Sound_Continue();
+		}
+#endif
 		SDL_INPUT_Mouse();
 		Atari800_Frame();
 		if (Atari800_display_screen)
